Exception: SparkConnect::IllegalArgumentError

Inherits:
Error
  • Object
show all
Defined in:
lib/spark_connect/errors.rb

Overview

Raised for invalid arguments passed to the public API before any request is sent to the server (mirrors PySpark’s analysis-time argument validation).